import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class UserManagerPanel extends JPanel {
	private MainWindow parent;

	private JList userList;
	private DefaultListModel userListModel;
	private FTPUser workingUser;

	private JList permissionList;
	private DefaultListModel permissionListModel;

	private JScrollPane userListScroll, permissionListScroll;

	private JTextField usernameEdit, passwordEdit;

	private JButton addUserButton, removeUserButton, saveButton, permAddButton, permRemoveButton;

	private JPanel userListPanel, rightPanel, buttonPanel, userInfoPanel, permissionsPanel, permissionsControlPanel;

	private ObjectSelectionWindow objectPicker;

	private JComboBox permDropDown;

	/**
	*	Inner class to hold permissions for the drop down list
	*/
	private class PermListItem {
		private String name, value;

		public PermListItem(String n, String v) {
			name = n;
			value = v;
		}

		public String getName() { return name; }
		public String getValue() { return value; }
		public String toString() { return name; }
	}

	private PermListItem[] permList = {
		new PermListItem("View hidden file", "ViewHiddenFilesPermission"),
		new PermListItem("View protected file", "ViewProtectedFilesPermission")
	};

	public UserManagerPanel(MainWindow p) {
			parent = p;

			objectPicker = new ObjectSelectionWindow(parent, parent.getServer().getRoot());

			addUserButton 		= new JButton("Add User");
			removeUserButton 	= new JButton("Remove User");
			saveButton		 	= new JButton("Save Changes");

			permDropDown = new JComboBox(permList);
			permDropDown.setMaximumSize(new Dimension(10000, permDropDown.getPreferredSize().height));

			userListModel = new DefaultListModel();

			userList = new JList(userListModel);
			userList.setMinimumSize(new Dimension(200, 100));
			userList.setPreferredSize(userList.getMinimumSize());
			userListScroll = new JScrollPane(userList);

			userListPanel = new JPanel();
			userListPanel.setLayout(new BoxLayout(userListPanel, BoxLayout.X_AXIS));
			userListPanel.add(new JScrollPane(userList));

			permissionListModel = new DefaultListModel();

			permissionList = new JList(permissionListModel);
			permissionList.setMinimumSize(new Dimension(300, 100));
			permissionListScroll = new JScrollPane(permissionList);

			permAddButton = new JButton("Add permission");
			permAddButton.setMaximumSize(new Dimension(10000, permAddButton.getPreferredSize().height));
			permRemoveButton = new JButton("Remove permission");
			permRemoveButton.setMaximumSize(new Dimension(10000, permRemoveButton.getPreferredSize().height));

			permissionsControlPanel = new JPanel();
			permissionsControlPanel.setLayout(new BoxLayout(permissionsControlPanel, BoxLayout.X_AXIS));
			permissionsControlPanel.add(permRemoveButton);
			permissionsControlPanel.add(permDropDown);
			permissionsControlPanel.add(permAddButton);

			permissionsPanel = new JPanel();
			permissionsPanel.setLayout(new BoxLayout(permissionsPanel, BoxLayout.Y_AXIS));
			permissionsPanel.add(permissionsControlPanel);
			permissionsPanel.add(permissionListScroll);

			buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			buttonPanel.add(addUserButton);
			buttonPanel.add(Box.createHorizontalGlue());
			buttonPanel.add(saveButton);
			buttonPanel.add(Box.createHorizontalGlue());
			buttonPanel.add(removeUserButton);

			usernameEdit = new JTextField(20);
			usernameEdit.setMaximumSize(new Dimension(10000, usernameEdit.getPreferredSize().height));
			usernameEdit.setMinimumSize(usernameEdit.getPreferredSize());

			passwordEdit = new JTextField(20);
			passwordEdit.setMaximumSize(new Dimension(10000, usernameEdit.getPreferredSize().height));
			passwordEdit.setMinimumSize(usernameEdit.getPreferredSize());

			userInfoPanel = new JPanel();
			userInfoPanel.setLayout(new GridLayout(2, 2));
			userInfoPanel.add(new JLabel("User name:"));
			userInfoPanel.add(usernameEdit);
			userInfoPanel.add(new JLabel("Password:"));
			userInfoPanel.add(passwordEdit);
			userInfoPanel.setMaximumSize(new Dimension(10000, userInfoPanel.getPreferredSize().height));

			rightPanel = new JPanel();
			rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
			rightPanel.add(userInfoPanel);
			rightPanel.add(Box.createVerticalGlue());
			rightPanel.add(permissionsPanel);
			rightPanel.add(Box.createVerticalGlue());
			rightPanel.add(buttonPanel);

			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			add(userListPanel);
			add(rightPanel);

			readUsers();

			userList.addListSelectionListener(new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent event) {
						Vector theseperms;
						int c;

						if (!event.getValueIsAdjusting()) {
							if (((JList)event.getSource()).getSelectedIndex() >= 0) {
								usernameEdit.setText(((FTPUser)(((JList)event.getSource()).getSelectedValue())).getUsername());
								passwordEdit.setText(((FTPUser)(((JList)event.getSource()).getSelectedValue())).getPassword());

								permissionListModel.removeAllElements();

								theseperms = ((FTPUser)(((JList)event.getSource()).getSelectedValue())).getPermissions();
								for(c=0;c<theseperms.size();c++) {
									permissionListModel.addElement(((FTPPermission)(theseperms.elementAt(c))).completePermission());
								}

								workingUser = (FTPUser)(((JList)event.getSource()).getSelectedValue());
							}
						}
					}
				});

			saveButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						StringTokenizer st;
						String pstring, astring;
						int d;

						workingUser.clearPermissions();
						for(d=0;d<permissionListModel.getSize();d++) {
							st = new StringTokenizer(permissionListModel.getElementAt(d).toString(), ":", false);

							pstring = st.nextToken();
							if (st.hasMoreTokens()) {
								astring = st.nextToken();
							} else {
								astring = "";
							}

							workingUser.addPermission(new FTPPermission(pstring, astring, 0));
						}

						writeUsers();
					}
				});

			permAddButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						if (usernameEdit.getText().trim().equals("")) {
							JOptionPane.showMessageDialog(parent, "You must select a user before adding permissions.",
														  "Error", JOptionPane.ERROR_MESSAGE);
							return;
						}
						objectPicker.showSelf();
						if (objectPicker.getOk()) {
							permissionListModel.addElement(((PermListItem)(permDropDown.getSelectedItem())).getValue() +
														   ":" + objectPicker.getPickedObject().completeObjectPath());
						}
					}
				});

			permRemoveButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						int selectedPerm;

						selectedPerm = permissionList.getSelectedIndex();
						if (selectedPerm < 0) {
							JOptionPane.showMessageDialog(parent, "You must select a permission before you can remove it.",
														  "Error", JOptionPane.ERROR_MESSAGE);
							return;
						}
						permissionListModel.removeElementAt(selectedPerm);
					}
				});

			usernameEdit.addFocusListener(new FocusAdapter() {
					public void focusLost(FocusEvent event) {
						FTPUser currentUser = (FTPUser)userList.getSelectedValue();
						currentUser.setUsername(usernameEdit.getText());
					}
				});

			passwordEdit.addFocusListener(new FocusAdapter() {
					public void focusLost(FocusEvent event) {
						FTPUser currentUser = (FTPUser)userList.getSelectedValue();
						currentUser.setPassword(passwordEdit.getText());
					}
				});

			addUserButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						userListModel.addElement(new FTPUser("(newuser)", new Long(System.currentTimeMillis()).toString()));
					}
				});

			removeUserButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						int selectedUser;

						selectedUser = userList.getSelectedIndex();
						userListModel.removeElementAt(selectedUser);
					}
				});
	}

	private void readUsers() {
		int currentLine = 1;

		BufferedReader users = null;
		String line = null, user = null, password = null;
		StringTokenizer st = null;

		FTPUser userObject;

		try {
			users = new BufferedReader(new FileReader(new File("users")));
		} catch (Exception fileException) {
			System.out.println("Error reading users file (" + fileException + ")");
		}

		try {
			while ((line = users.readLine()) != null) {
				if (line.trim().equalsIgnoreCase("")) continue;

				st = new StringTokenizer(line, ":", false);
				try {
					user = st.nextToken();
					password = st.nextToken();

					userObject = new FTPUser(user, password);
					userObject.getFilePermissions();

					userListModel.addElement(userObject);
				} catch (Exception stringException) {
					System.out.println("Error in users file, on or near line " + currentLine);
				}
				currentLine++;
			}
		} catch (Exception readingException) {
			System.out.println("Error reading the users file, at or near line " + currentLine);
		}
	}

	private void writeUsers() {
		int c, d;

		Vector permissions = null;
		PrintWriter pw = null;
		PrintWriter perms = null;

		FTPUser currentUser;

		try {
			pw = new PrintWriter(new FileWriter(new File("users")));
			perms = new PrintWriter(new FileWriter(new File("usersecurity")));
		} catch (Exception fileError) {
			System.out.println("Error saving users/usersecurity file");
		}

		System.out.println("Writting users and usersecurity files");

		for(c=0;c<userListModel.getSize();c++) {
			currentUser = (FTPUser)userListModel.elementAt(c);
			pw.println(currentUser.getUsername() + ":" + currentUser.getPassword());
			permissions = currentUser.getPermissions();
			for(d=0;d<permissions.size();d++) {
				perms.println(currentUser.getUsername() + ":" + ((FTPPermission)(permissions.elementAt(d))).completePermission());
			}
		}

		try {
			pw.close();
			perms.close();
		} catch (Exception closeError) {
			System.out.println("Error closing users/usersecurity file");
		}
	}
}